var prevDOM = null;

chrome.runtime.onMessage.addListener(
    function OnGetMessage(message, sender, sendResponse) 
    {
        try {

            if (message.Action == "GetElementAtCursor") {
                if (prevDOM != null) {

                    var resp =
                    {
                        CurrentElement: prevDOM.tagName,
                        Rect: prevDOM.getBoundingClientRect()
                    }
                    var framePosition = currentFrameAbsolutePosition();
                    
                    sendResponse(resp);
                }
            }
            else if (message.Action == "GetElementScreenBounds") {
                var selector = message.Parameters[1];
                var element = document.querySelector(selector);
                var dpi = window.devicePixelRatio;
                var borderwidth = 8;

                var viewport = {
                    x: (window.top.screenX * dpi) + borderwidth,
                    y: (window.top.screenY + (window.top.outerHeight - window.top.innerHeight)) * dpi - borderwidth,
                    width: window.top.outerWidth * 1.25 - (2 * borderwidth),
                    height: window.top.outerHeight * 1.25 - borderwidth,
                    dpi: dpi
                }

                var isMaximized = (screen.availWidth - window.top.outerWidth === 0);
                if (isMaximized == true) {
                    viewport.x = viewport.x - 7;
                    viewport.y = viewport.y + 8;
                }

                var resp =
                {
                    CurrentElement: "Element_Not_Found",
                    Bounds: ""
                };

                if (element != null) {
                    var bnd = element.getBoundingClientRect();
                    var scale = window.devicePixelRatio;
                    var textContent = element.textContent.substring(0, 1024); //trim text contents to max size of 1024 chars for performance reasons

                    var framePosition = currentFrameAbsolutePosition();
                    
                    var resp =
                    {
                        CurrentElement: element.tagName,
                        Bounds:
                        {
                            x: bnd.x + framePosition.x,
                            y: bnd.y + framePosition.y,
                            width: bnd.width,
                            height: bnd.height
                        },
                        Viewport: viewport,
                        Text: textContent,
                    };

                    if (bnd.x != 0 && bnd.width != 0 && bnd.height != 0 && bnd.y != 0) 
                        sendResponse(resp);
                }
            }
            else if (message.Action == "GetElementBySelector") {
                var selector = message.Parameters[1];
                var element = document.querySelector(selector);
                
                var resp =
                {
                    CurrentElement: "Element_Not_Found",
                    Bounds: ""
                };

                if (element != null) {
                    var attr = Array.prototype.map.call(element.attributes, function (a) { return { attribute: a.nodeName, value: a.value }; });

                    var resp =
                    {
                        CurrentElement: element.tagName,
                        Bounds: element.getBoundingClientRect(),
                        Text: element.textContent,
                        Attributes: attr
                    };
                    
                    sendResponse(resp);
                }
            }
            else if (message.Action == "GetElementDOM") {
                var selector = message.Parameters[1];
                var element = document.querySelector(selector);

                var resp =
                {
                    CurrentElement: "Element_Not_Found",
                    Bounds: ""
                };

                if (element != null) {
                    var resp =
                    {
                        CurrentElement: element.tagName,
                        DOM: element.outerHTML
                    };

                    sendResponse(resp);
                }
            }

            else if (message.Action == "RunJavaScript") {
                var jscode = message.Parameters[1];
                var script = document.createElement('script');
                var code = document.createTextNode(jscode);
                script.appendChild(code);
                (document.body || document.head).appendChild(script);

                sendResponse({ RunJavaScript: "Injected" });
            }
            else if (message.Action == "ScrollToElement") {
                var selector = message.Parameters[1];
                var element = document.querySelector(selector);

                var resp =
                {
                    CurrentElement: "Element_Not_Found",
                    Rect: ""
                };

                if (element != null) {

                    var bounding = element.getBoundingClientRect();
                    var isInView = (bounding.top >= 0 && bounding.left >= 0 && bounding.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
                        bounding.right <= (window.innerWidth || document.documentElement.clientWidth));

                    if (isInView === false) {
                        element.scrollIntoView(
                            {
                                behavior: 'auto',
                                block: 'center',
                                inline: 'center'
                            });

                        var resp =
                        {
                            CurrentElement: "Element_Scrolled_Into_View",
                            Rect: bounding,
                        };
                    }

                    sendResponse(resp);
                }
            }
            else {
                var resp =
                {
                    Response: "Unknown_command",
                };
                sendResponse(resp);
            }
        }
        catch (e)
        {
            var msg = "Error executing " + message.Action + " - " + e.message;
            console.log(msg);
            var resp =
            {
                Response: "Error",
                Action: message,
                Reason: msg
            };
            sendResponse(resp);
        }

        //return true;
    }
);

window.addEventListener('load', (event) => 
{
    document.documentElement.addEventListener('mouseleave', function(event) 
    {
        var update = {
            MessageType: "MouseHoverEvent",
            Element: null,
        }

        try {
            chrome.runtime.sendMessage(
                update,
                function (response) { }
            );
        }
        catch (e) {
            console.log("Could not send information to Tailent Extension." + e.message);
        }
    });
});

window.addEventListener('mousemove', function(event) 
{
    let srcElement = event.target;

    if (prevDOM == srcElement)
        return;

    prevDOM = srcElement;
    var frameOffset = currentFrameAbsolutePosition();

    var selector = 
    {
        //Page: document.URL,
        Page: window.top.document.URL, //use top level url
        DOM: getControlIdFull(srcElement)
    }

    var boundingRectangle = srcElement.getBoundingClientRect();
    boundingRectangle.x += frameOffset.x;
    boundingRectangle.y += frameOffset.y;

    var update = {
        MessageType: "MouseHoverEvent",
        Bounds: boundingRectangle,
        Element: srcElement.tagName,
        Selector: selector,
        Frame: frameOffset
    }

    try {
        chrome.runtime.sendMessage(
            update,
            function (response) { }
        );
    }
    catch (e)
    {
        console.log("Could not send information to Tailent Extension." + e.message);
    }
});

function GetElementIndex(input)
 {
    const parent = input.parentNode
    if (!parent) {
      return -1;
    }
  
    let child = parent.firstChild
    if (!child) {
      return -1;
    }
  
    let i = 0
    while (child) {
      if (child.nodeType === Node.ELEMENT_NODE) {
        i++
      }
  
      if (child === input) {
        break
      }
  
      child = child.nextSibling
    }
  
    return i
  }

function getControlIdFull (el) {
    if (!(el instanceof Element)) 
        return;
    var path = [];
    while (el.nodeType === Node.ELEMENT_NODE) 
    {
        var tagName = el.nodeName.toLowerCase();
        var tagId = el.id;
        var classes = [].slice.call(el.classList);
        var attr = Array.prototype.map.call(el.attributes, function(a) { return { attribute: a.nodeName, value: a.value};});

        var sib = el, nth = 1;
        while (sib = sib.previousElementSibling) 
        {
            if (sib.nodeName.toLowerCase() == tagName)
                nth++;
        }

        if (nth == 1)
            nth = -1;
        else
            nth = GetElementIndex(el);

        selector = 
        {
            element: tagName,
            id: tagId,
            cls: classes,
            index: nth,
            attributes: attr
        };

        path.unshift(selector);
        el = el.parentNode;
    }
    return JSON.stringify(path);
}

function currentFrameAbsolutePosition() {
    let currentWindow = window;
    let currentParentWindow;
    let positions = [];
    let rect;

    while (currentWindow !== window.top) {
        currentParentWindow = currentWindow.parent;
        for (let idx = 0; idx < currentParentWindow.frames.length; idx++)
            if (currentParentWindow.frames[idx] === currentWindow) {
                for (let frameElement of currentParentWindow.document.getElementsByTagName('iframe')) {
                    if (frameElement.contentWindow === currentWindow) {
                        rect = frameElement.getBoundingClientRect();
                        positions.push({ x: rect.x, y: rect.y });
                    }
                }
                currentWindow = currentParentWindow;
                break;
            }
    }
    return positions.reduce((accumulator, currentValue) => {
        return {
            x: accumulator.x + currentValue.x,
            y: accumulator.y + currentValue.y
        };
    }, { x: 0, y: 0 });
}

