var Port = null;
var CurrentHoveredTab = null;
var CurrentHoveredElement = null;
var frameOffset = null;
var TabStatus = [];

chrome.runtime.onMessage.addListener(function (message, sender, sendResponse) {
    if (message.MessageType == "MouseHoverEvent") {
        if (message.Element != null) {
            CurrentHoveredTab = sender.tab;
            CurrentHoveredElement = message;
        }
        else {
            CurrentHoveredTab = null;
            CurrentHoveredElement = null;
        }
    }
    sendResponse("ACK");
});

chrome.runtime.onStartup.addListener()

chrome.runtime.onInstalled.addListener(function (details) {
    chrome.tabs.query({}, function (tabs) {
        for (var i in tabs) {
            if ((tabs[i].url && tabs[i].url.indexOf("chrome://") === 0) == false) {
                //chrome.tabs.reload(tabs[i].id, {});
            }
        }
    });
});

var NativeComm = function () {

    console.log('TAP Chrome Extension started.');
    
    var AppName = "com.tap.automation";
    Port = chrome.runtime.connectNative(AppName);

    //top level events
    chrome.tabs.onUpdated.addListener(function (tabId, info, tab) {
        var prevItem = TabStatus[tabId];
        var _url = tab.url;
        var _status = info.status;

        if (prevItem == null) {
            prevItem = { url: "", status: "" };
        }

        if (_url != null)
            prevItem.url = _url;

        if (_status != null)
            prevItem.status = _status;

        TabStatus[tabId] = prevItem;
    });

    chrome.tabs.onRemoved.addListener(function (tabId, removeInfo)
    {
        if (TabStatus[tabId] != null) {
            TabStatus.splice(tabId, 1);
        }
    });

    Port.onMessage.addListener(function (message) {
        //console.log("Received message " + message.Action);

        if (message.Action == "GetElementAtCursor") {
            var resp;
            if (CurrentHoveredElement != null) {
                resp =
                {
                    CurrentElement: CurrentHoveredElement,
                }
            }
            else {
                resp = {
                    CurrentElement: "no_element",
                }
            }
            Port.postMessage(resp);
        }

        else if (message.Action == "GetElementScreenBounds") {
            var tabUrl = message.Parameters[0];

            chrome.tabs.query({}, function (tabs) {
                var filteredTabs = tabs.filter(x => x.url.includes(tabUrl));
                if (filteredTabs.length > 0) {

                    chrome.tabs.sendMessage(filteredTabs[0].id, message, 
                        function (response) {
                            if (Port != null) {
                                Port.postMessage(response);
                            }
                        });
                }
                else {
                    resp = { CurrentElement: "Element_Not_Found" };
                    Port.postMessage(resp);
                }
            });
        }
        else if (message.Action == "GetElementBySelector") {
            var tabUrl = message.Parameters[0];

            chrome.tabs.query({}, function (tabs) {
                var filteredTabs = tabs.filter(x => x.url.includes(tabUrl));
                if (filteredTabs.length > 0)
                    chrome.tabs.sendMessage(filteredTabs[0].id, message,
                        function (response) {
                            if (Port != null) {
                                Port.postMessage(response);
                            }
                        });
                else {
                    resp = { CurrentElement: "Element_Not_Found" };
                    Port.postMessage(resp);
                }
            });
        }
        else if (message.Action == "GetElementDOM") {
            var tabUrl = message.Parameters[0];

            chrome.tabs.query({}, function (tabs) {
                var filteredTabs = tabs.filter(x => x.url.includes(tabUrl));
                if (filteredTabs.length > 0)
                    chrome.tabs.sendMessage(filteredTabs[0].id, message,
                        function (response) {
                            if (Port != null) {
                                Port.postMessage(response);
                            }
                        });
                else {
                    resp = { CurrentElement: "Element_Not_Found" };
                    Port.postMessage(resp);
                }
            });
        }
        else if (message.Action == "RunJavaScript") {
            var tabUrl = message.Parameters[0];

            chrome.tabs.query({}, function (tabs) {
                var filteredTabs = tabs.filter(x => x.url.includes(tabUrl));
                if (filteredTabs.length > 0) {
                    chrome.tabs.sendMessage(filteredTabs[0].id, message,
                        function (response) {
                            if (Port != null) {
                                Port.postMessage(response);
                            }
                        });
                }
                else {
                    Port.postMessage({ RunJavaScript: "Window_Not_Found" });
                }
            });
        }
        else if (message.Action == "ScrollToElement") {
            var tabUrl = message.Parameters[0];

            chrome.tabs.query({}, function (tabs) {
                var filteredTabs = tabs.filter(x => x.url.includes(tabUrl));
                if (filteredTabs.length > 0) {
                    chrome.tabs.sendMessage(filteredTabs[0].id, message,
                        function (response) {
                            if (Port != null) {
                                Port.postMessage(response);
                            }
                        });
                }
                else {
                    Port.postMessage({ ScrollToElement: "Window_Not_Found" });
                }
            });
        }
        else if (message.Action == "ActivateTab") {
            var tabUrl = message.Parameters[0];

            chrome.tabs.query({}, function (tabs) {
                var filteredTabs = tabs.filter(x => x.url.includes(tabUrl));
                if (filteredTabs.length > 0) {
                    var updateProperties = { active: true };
                    chrome.tabs.update(filteredTabs[0].id, updateProperties, (tab) => {
                        Port.postMessage({ ActivateTab: "Command_Executed" });
                    });
                }
                else {
                    Port.postMessage({ ActivateTab: "TAB_Not_Found" });
                }
            });
        }
        else if (message.Action == "ActivateWindow") {
            var tabUrl = message.Parameters[0];
            
            chrome.tabs.query({}, function (tabs) {
                var filteredTabs = tabs.filter(x => x.url.includes(tabUrl));
                if (filteredTabs.length > 0) {
                    var windowId = filteredTabs[0].windowId;
                    var windowOptions = { focused: true };

                    chrome.windows.update(windowId, windowOptions, (win) => {
                        console.log("Focusing window with id: " + windowId);
                        
                        var updateProperties = { active: true };
                        chrome.tabs.update(filteredTabs[0].id, updateProperties, (tab) => { });
                        Port.postMessage({ ActivateWindow: "Command_Executed" });
                    });
                    //});
                }
                else {
                    Port.postMessage({ ActivateTab: "TAB_Not_Found" });
                }
            });
        }
        else if (message.Action == "GetWindowProperties") {
            var tabUrl = message.Parameters[0];

            chrome.tabs.query({ url: tabUrl }, function (tabs) {
                if (tabs.length > 0) {
                    chrome.windows.get(tabs[0].windowId, (win) => {
                        var props = JSON.stringify(win);
                        console.log(props);
                        Port.postMessage(
                            { GetWindowProperties: props }
                        );
                    });
                }
                else {
                    Port.postMessage({ GetWindowProperties: "TAB_Not_Found" });
                }
            });
        }
        else if (message.Action == "SetDownloadBar") {
            try {
                var flag = (message.Parameters[0].toLowerCase() == 'true');
                chrome.downloads.setShelfEnabled(flag);
                Port.postMessage({ DisableDownloadBar: flag });
            }
            catch (e) {
                Port.postMessage({ DisableDownloadBar: "Failed_" + e.message });
            }
        }
        else if (message.Action == "GetTabStatus") {
            var tabUrl = message.Parameters[0];

            chrome.tabs.query({}, function (tabs) {
                var filteredTabs = tabs.filter(x => x.url.includes(tabUrl));
                if (filteredTabs.length > 0) {

                    var id = filteredTabs[0].id;
                    if (TabStatus[id] != null) {
                        Port.postMessage({ GetTabStatus: TabStatus[id].status });
                    }
                    else {
                        Port.postMessage({ GetTabStatus: "TAB_Not_Found" });
                    }
                }
                else {
                    Port.postMessage({ GetTabStatus: "TAB_Not_Found" });
                }
            });
        }
        else if (message.Action == "Ping") {
            Port.postMessage({ Ping: "Extension_Active" });
        }

        //return true;
    });

    Port.onDisconnect.addListener(function (message) {
    });

}();
